/********************************************************************************************
 *     LEGAL DISCLAIMER
 *
 *     (Header of MediaTek Software/Firmware Release or Documentation)
 *
 *     BY OPENING OR USING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 *     THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE") RECEIVED
 *     FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON AN "AS-IS" BASIS
 *     ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES, EXPRESS OR IMPLIED,
 *     INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
 *     A PARTICULAR PURPOSE OR NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY
 *     WHATSOEVER WITH RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 *     INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK
 *     ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
 *     NOT BE RESPONSIBLE FOR AgNY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S SPECIFICATION
 *     OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
 *
 *     BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE LIABILITY WITH
 *     RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION,
 *     TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE
 *     FEES OR SERVICE CHARGE PAID BY BUYER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 *     THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE WITH THE LAWS
 *     OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF LAWS PRINCIPLES.
 ************************************************************************************************/
/*
*
* \par Description
*   The function bodies of mt85 system
*
*/

#include <pthread.h>

#include <sys/ioctl.h>
#include <sys/mman.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <assert.h>

#include <asm/types.h>

#include <directfb.h>

#include <direct/mem.h>
#include <direct/memcpy.h>
#include <direct/messages.h>
#include <direct/system.h>
#include <direct/util.h>

#include <fusion/arena.h>
#include <fusion/fusion.h>
#include <fusion/reactor.h>
#include <fusion/shmalloc.h>

#include <core/core.h>
#include <core/layers.h>
#include <core/gfxcard.h>
#include <core/screens.h>
#include <core/surface_pool.h>

#include <misc/conf.h>
#include <misc/util.h>

#include "mt85.h"
#include "mt85_vt.h"

#include <core/core_system.h>

#include "../../bd_inc/mt85xx_fb.h"

DFB_CORE_SYSTEM( mt85sys )

MT85 *dfb_mt85 = NULL;

MULTI_POOL mt85_multi_pool;

#if DFB_SUPPORT_GFX_ADAPTER
UINT32           _h_mt85_Ticket = 0;
UINT32           _gptCmdbufDrv  = 0;
UINT32           _gui4CmdbufLen = 0;

static INT32  mt85AdapterCmdBufGetCmdBuf(INT32 fd)
{
    MT85_ADAPTER_CMDBUF_T*      ptCmdBuff = NULL;
    INT32                       i4Ret = 0;
    GFX_DRV_CMDBUF_PRIORITY     u4Priority = GFX_DRV_CMDBUF_PRIORITY_HIGH;
    mt8530fb_mmap_method      mapmtd;

    ptCmdBuff = (MT85_ADAPTER_CMDBUF_T *)malloc(sizeof(MT85_ADAPTER_CMDBUF_T));
    assert(ptCmdBuff != NULL);

    ptCmdBuff->fg_compress = FALSE;
    ptCmdBuff->u4GfxPriority = (UINT32)u4Priority;

    if (ioctl(fd, FBIO_GETCMDBUFLEN, &_gui4CmdbufLen) < 0)
    {
        D_PERROR( "MediaTek/Driver: FBIO_GETCMDBUFLEN failed!\n" );
    }
    printf("%s: u4CmdbufLen = 0x%X\n", __FUNCTION__, _gui4CmdbufLen);
    if (0 == _gui4CmdbufLen)
    {
       free((VOID*)ptCmdBuff);
       
        return -1;
    }

    mapmtd = FB_MMAP_GFXCMDBUF;
    if (ioctl(fd, FBIO_SETMMAPMETHOD, &mapmtd) < 0)
    {
        D_PERROR( "MediaTek/Driver: FBIO_SETMMAPMETHOD failed!\n" );
    }
    _gptCmdbufDrv = (UINT32)mmap(NULL, _gui4CmdbufLen, PROT_READ | PROT_WRITE, MAP_SHARED, dfb_mt85->fd, 0);
    ptCmdBuff->prGfxMwCmdBuf = (GFX_LINUX_CMDBUF_T*)(_gptCmdbufDrv + sizeof(GFX_DRV_CMDBUF_T));
    _h_mt85_Ticket = (UINT32)(ptCmdBuff);
    ptCmdBuff->u4Private = _h_mt85_Ticket;

    return 0;
}

static INT32 mt85AdapterCmdBufRelCmdBuf(INT32 fd)
{
    INT32                   i4Ret = -1;
    MT85_ADAPTER_CMDBUF_T*  ptCmdBuff = NULL;

    ptCmdBuff = (MT85_ADAPTER_CMDBUF_T* )_h_mt85_Ticket;

    if (_gptCmdbufDrv || _gui4CmdbufLen)
    {
        i4Ret = munmap( (void*)_gptCmdbufDrv, _gui4CmdbufLen );
        if (!i4Ret)
        {
            _gptCmdbufDrv = 0;
            /*printf("*************Unmap DirectFB GFX command buffer success.\n");*/
        }
        else
        {
            printf("*************Unmap DirectFB GFX command buffer failed! ERROR_ID = %d\n", i4Ret);
            assert(0);
        }
        if (ioctl(fd, FBIO_RELCMDBUF, ptCmdBuff->prGfxMwCmdBuf) < 0)
        {
            D_PERROR( "MediaTek/Driver: FBIO_RELCMDBUF failed!\n" );
        }

        free((VOID*)ptCmdBuff);
    }

    return i4Ret;
}

INT32 mt85AdapterCmdBufInit(INT32 fd)
{
    if (0 != mt85AdapterCmdBufGetCmdBuf(fd))
    {
        return -1;
    }

    return 0;
}

INT32 mt85AdapterCmdBufUninit(INT32 fd)
{
    if (0 != mt85AdapterCmdBufRelCmdBuf(fd))
    {
        return -1;
    }

    return 0;
}
#endif

#define FBD_DFB_DEBUG_MEM_SIZE      4096    // 4k

//void *_fb_dfb_debug_info; //modified by Yaoyong Ju

inline void *mt85_debuginfo(void)
{
    return _fb_dfb_debug_info;
}

static int mt85_map_debuginfo(unsigned int fd)
{
    mt8530fb_mmap_method    mapmtd;

    // set mmap method
    mapmtd = FB_MMAP_DEBUGINFO;
    if (ioctl(fd, FBIO_SETMMAPMETHOD, &mapmtd) < 0)
    {
        D_PERROR( "MediaTek/Driver: FBIO_SETMMAPMETHOD failed!\n" );
    }
    _fb_dfb_debug_info = mmap(NULL, FBD_DFB_DEBUG_MEM_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);

    return 0;
}

static int mt85_unmap_debuginfo(unsigned int fd)
{
    if (_fb_dfb_debug_info)
    {
        if (munmap(_fb_dfb_debug_info, FBD_DFB_DEBUG_MEM_SIZE) == 0)
        {
            _fb_dfb_debug_info = NULL;
            D_INFO("[DirectFB]Unmap DirectFB debug info memory success.\n");
        }
        else
        {
            D_INFO("[DirectFB]Unmap DirectFB debug info memory failed!\n");
            assert(0);
        }
    }

    return 0;
}

/**********************************************************************************************************************/
//add by mao zhong for pana
DFBResult
system_get_offset_by_ptr(unsigned long ptr , unsigned long *offset)
{
    D_INFO("Enter in %s:%d\n",__FUNCTION__,__LINE__);

    int i =0;
    unsigned long _offset = 0;
    unsigned long _addr = 0;
    unsigned long _size = 0;
    *offset = -1;

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        _addr = (unsigned long)mt85_multi_pool.addr[i];
        _size = (unsigned long)mt85_multi_pool.size[i];
        if ((ptr >= _addr) && (ptr <= (_addr + _size)))
        {
            *offset = _offset + (ptr -_addr);
            break;
        }
        _offset += _size;
    }
    if (-1 == *offset)
        return DFB_FAILURE;

    DFB_MEM_LOG("[DirectFB][Memory]%s ptr = %ld,offset = 0x%x\n",__FUNCTION__,ptr,(unsigned int)*offset);

    return DFB_OK;
}

DFBResult
system_get_ptr_by_offset(unsigned long offset ,unsigned long *ptr)
{
    D_INFO("Enter in %s:%d\n",__FUNCTION__,__LINE__);

    int i = 0;
    unsigned long _offset = 0;

    unsigned long _addr = 0;
    unsigned long _size = 0;
    *ptr = -1;

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        _addr = (unsigned long)mt85_multi_pool.addr[i];
        _size = (unsigned long)mt85_multi_pool.size[i];
        if ((offset >= _offset) && (offset <= (_offset + _size)))
        {
            *ptr = _addr + (offset-_offset);
            break;
        }
        _offset += _size;

    }
    if (-1 == *ptr)
        return DFB_FAILURE;

    DFB_MEM_LOG("[DirectFB][Memory]%s offset = %ld,Ptr = 0x%x\n",__FUNCTION__,offset,(unsigned int)*ptr);

    return DFB_OK;

}

static void
system_get_info( CoreSystemInfo *info )
{
    info->type = CORE_ANY;
    info->caps = CSCAPS_ACCELERATION;

    snprintf( info->name, DFB_CORE_SYSTEM_INFO_NAME_LENGTH, "MediaTek 85xx" );
}

static DFBResult
system_initialize( CoreDFB *core, void **data )
{
    DFBResult                 ret = DFB_INIT;
    struct fb_var_screeninfo  var;
    FusionSHMPoolShared      *pool;
    CoreScreen               *screen;
    MT85Shared               *shared = NULL;
    int                       i = 0;
    mt8530fb_mmap_method      mapmtd;

    D_INFO( "MediaTek/System: %s: #%d\n", __FUNCTION__, __LINE__ );

    D_ASSERT( dfb_mt85 == NULL );

    pool = dfb_core_shmpool( core );

    dfb_mt85 = D_CALLOC( 1, sizeof(MT85) );
    if (!dfb_mt85)
        return D_OOM();

    dfb_mt85->fd = -1;

    shared = (MT85Shared*) SHCALLOC( pool, 1, sizeof(MT85Shared) );
    if (!shared)
    {
        ret = D_OOSHM();
        goto error;
    }

    shared->shmpool = pool;

    fusion_arena_add_shared_field( dfb_core_arena( core ), "mt85", shared );

    dfb_mt85->core   = core;
    dfb_mt85->shared = shared;

    dfb_mt85->fd = direct_try_open( "/dev/graphics/fb0", "/dev/fb/0", O_RDWR, true );
    if (dfb_mt85->fd < 0)
        goto error;

    if (dfb_config->vt)
    {
        ret = dfb_vt_initialize();
        if (ret)
            goto error;
    }

    ret = DFB_INIT;

    /* Initialization of mt85fb */
    var.xres           = MT85_PRIMARY_DEFAULT_WIDTH;
    var.yres           = MT85_PRIMARY_DEFAULT_HEIGHT;
    var.xres_virtual   = MT85_PRIMARY_DEFAULT_WIDTH;
    var.yres_virtual   = MT85_PRIMARY_DEFAULT_HEIGHT * 4;
    var.xoffset        = 0;
    var.yoffset        = 0;
    var.bits_per_pixel = 32;
    var.vmode = GFX_COLORMODE_ARGB_D8888;

    if (ioctl( dfb_mt85->fd, FBIO_INIT, &var ) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIO_INIT failed!\n" );
        goto error;
    }

    if (ioctl( dfb_mt85->fd, FBIOGET_FSCREENINFO, &dfb_mt85->fix ) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIOGET_FSCREENINFO failed!\n" );
        goto error;
    }


    if (ioctl( dfb_mt85->fd, FBIO_GETIMAGEBUFFER, &dfb_mt85->imagebuf ) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIO_GETIMAGEBUFFER failed!\n" );
        goto error;
    }

    /* Map the framebuffer */

    printf("UUU:CONFIG_DIRECTFB_MULTI_MEM_POOL\n");

    if (ioctl( dfb_mt85->fd, FBIO_GETMULTIPOOL, &mt85_multi_pool) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIO_GETMULTIPOOL failed!\n" );
        goto error;
    }

    if (mt85_multi_pool.num == 0)
    {
        D_PERROR( "DirectFB/MT85: Get memory failed!\n" );
        goto error;
    }

    // set mmap method
    mapmtd = FB_MMAP_MEMPOOL;
    if (ioctl(dfb_mt85->fd, FBIO_SETMMAPMETHOD, &mapmtd) < 0)
    {
        D_PERROR( "MediaTek/Driver: FBIO_SETMMAPMETHOD failed!\n" );
    }
    printf("[5J] %s -> map memory pool\n", __FUNCTION__);
    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        mt85_multi_pool.addr[i] = mmap( NULL, mt85_multi_pool.size[i] + i * 0x1000, PROT_READ | PROT_WRITE, MAP_SHARED, dfb_mt85->fd, 0 );

        if (mt85_multi_pool.addr[i] == MAP_FAILED)
        {
            D_PERROR( "DirectFB/MT85: Could not mmap pool!\n");
            mt85_multi_pool.addr[i] = 0;
            goto error;
        }
    }

    dfb_mt85->mem = 0;

    dfb_surface_pool_initialize( core, &mt85SurfacePoolFuncs, &dfb_mt85->shared->pool );

    *data = dfb_mt85;

    printf("[5J] %s -> map cmdbuf\n", __FUNCTION__);
#if DFB_SUPPORT_GFX_ADAPTER
    mt85AdapterCmdBufInit(dfb_mt85->fd);
#endif

    printf("[5J] %s -> map debug info\n", __FUNCTION__);
    mt85_map_debuginfo(dfb_mt85->fd);

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [Pool %d]\n",__FUNCTION__,__LINE__,i);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [addr=0x%lx]\n",__FUNCTION__,__LINE__,(long unsigned int)mt85_multi_pool.addr[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [phys=0x%lx]\n",__FUNCTION__,__LINE__,(long unsigned int)mt85_multi_pool.phys[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [size=0x%lx]\n",__FUNCTION__,__LINE__,mt85_multi_pool.size[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [type=%d]\n",__FUNCTION__,__LINE__,mt85_multi_pool.type[i]);
    }

    return DFB_OK;


error:
    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        if (mt85_multi_pool.addr[i])
        {
            munmap(mt85_multi_pool.addr[i], mt85_multi_pool.size[i]);
            mt85_multi_pool.addr[i] = 0;
        }
    }

    if (shared)
        SHFREE( pool, shared );

    if (dfb_mt85->mem)
        munmap( dfb_mt85->mem, dfb_mt85->fix.smem_len );

    if (dfb_mt85->fd >= 0)
        close( dfb_mt85->fd );

    D_FREE( dfb_mt85 );
    dfb_mt85 = NULL;

    return ret;
}

static DFBResult
system_join( CoreDFB *core, void **data )
{
    DFBResult   ret = DFB_INIT;
    CoreScreen *screen;
    void       *shared;
    int                       i = 0;
    mt8530fb_mmap_method      mapmtd;

    D_INFO( "MediaTek/System: %s: #%d\n", __FUNCTION__, __LINE__ );

    D_ASSERT( dfb_mt85 == NULL );

    if (dfb_config->vt)
    {
        ret = dfb_vt_join();
        if (ret)
            return ret;
    }

    dfb_mt85 = D_CALLOC( 1, sizeof(MT85) );
    if (!dfb_mt85)
        return D_OOM();

    fusion_arena_get_shared_field( dfb_core_arena( core ), "mt85", &shared );

    dfb_mt85->core   = core;
    dfb_mt85->shared = shared;

    dfb_mt85->fd = direct_try_open( "/dev/graphics/fb0", "/dev/fb/0", O_RDWR, true );
    if (dfb_mt85->fd < 0)
        goto error;

    if (ioctl( dfb_mt85->fd, FBIOGET_FSCREENINFO, &dfb_mt85->fix ) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIOGET_FSCREENINFO failed!\n" );
        goto error;
    }

    if (ioctl( dfb_mt85->fd, FBIO_GETIMAGEBUFFER, &dfb_mt85->imagebuf ) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIO_GETIMAGEBUFFER failed!\n" );
        goto error;
    }


    if (ioctl( dfb_mt85->fd, FBIO_GETMULTIPOOL, &mt85_multi_pool) < 0)
    {
        D_PERROR( "DirectFB/MT85: FBIO_GETMULTIPOOL failed!\n" );
        goto error;
    }

    if (mt85_multi_pool.num == 0)
    {
        D_PERROR( "DirectFB/MT85: Get memory failed!\n" );
        goto error;
    }

    // set mmap method
    mapmtd = FB_MMAP_MEMPOOL;
    if (ioctl(dfb_mt85->fd, FBIO_SETMMAPMETHOD, &mapmtd) < 0)
    {
        D_PERROR( "MediaTek/Driver: FBIO_MAP_GFXCMDBUF failed!\n" );
    }
    printf("[5J] %s -> map memory pool\n", __FUNCTION__);
    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        mt85_multi_pool.addr[i] = mmap( NULL, mt85_multi_pool.size[i] + i * 0x1000, PROT_READ | PROT_WRITE, MAP_SHARED, dfb_mt85->fd, 0 );

        if (mt85_multi_pool.addr[i] == MAP_FAILED)
        {
            D_PERROR( "DirectFB/MT85: Could not mmap pool!\n");
            mt85_multi_pool.addr[i] = 0;
            goto error;
        }
    }
    dfb_mt85->mem = 0;

    dfb_surface_pool_join( core, dfb_mt85->shared->pool, &mt85SurfacePoolFuncs );

    *data = dfb_mt85;

    printf("[5J] %s -> map cmdbuf\n", __FUNCTION__);
#if DFB_SUPPORT_GFX_ADAPTER
    mt85AdapterCmdBufInit(dfb_mt85->fd);
#endif

    printf("[5J] %s -> map debug info\n", __FUNCTION__);
    mt85_map_debuginfo(dfb_mt85->fd);

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [Pool %d]\n",__FUNCTION__,__LINE__,i);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [addr=0x%lx]\n",__FUNCTION__,__LINE__,(long unsigned int)mt85_multi_pool.addr[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [phys=0x%lx]\n",__FUNCTION__,__LINE__,(long unsigned int)mt85_multi_pool.phys[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [size=0x%lx]\n",__FUNCTION__,__LINE__,mt85_multi_pool.size[i]);
        DFB_MEM_LOG("[DirectFB][Memory]%s:%d [type=%d]\n",__FUNCTION__,__LINE__,mt85_multi_pool.type[i]);
    }

    return DFB_OK;


error:
    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        if (mt85_multi_pool.addr[i])
        {
            munmap(mt85_multi_pool.addr[i], mt85_multi_pool.size[i]);
            mt85_multi_pool.addr[i] = 0;
        }
    }

    if (dfb_mt85->mem)
        munmap( dfb_mt85->mem, dfb_mt85->fix.smem_len );

    if (dfb_mt85->fd >= 0)
        close( dfb_mt85->fd );

    D_FREE( dfb_mt85 );
    dfb_mt85 = NULL;

    return ret;
}

static DFBResult
system_shutdown( bool emergency )
{
    DFBResult            ret;
    MT85Shared          *shared;
    FusionSHMPoolShared *pool;
    int                  i = 0;

    D_INFO( "MediaTek/System: %s: #%d\n", __FUNCTION__, __LINE__ );

    D_ASSERT( dfb_mt85 != NULL );

    shared = dfb_mt85->shared;

    D_ASSERT( shared != NULL );

    pool = shared->shmpool;

    D_ASSERT( pool != NULL );

    mt85_unmap_debuginfo(dfb_mt85->fd);

#if DFB_SUPPORT_GFX_ADAPTER
    mt85AdapterCmdBufUninit(dfb_mt85->fd);
#endif

    dfb_surface_pool_destroy( dfb_mt85->shared->pool );

    if (dfb_mt85->mem)
        munmap( dfb_mt85->mem, dfb_mt85->fix.smem_len );

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        if (mt85_multi_pool.addr[i])
        {
            munmap(mt85_multi_pool.addr[i], mt85_multi_pool.size[i]);
            mt85_multi_pool.addr[i] = 0;
        }
    }

    if (dfb_config->vt)
    {
        ret = dfb_vt_shutdown( emergency );
        if (ret)
            return ret;
    }

    close( dfb_mt85->fd );

    SHFREE( pool, shared );
    D_FREE( dfb_mt85 );
    dfb_mt85 = NULL;

    return DFB_OK;
}

static DFBResult
system_leave( bool emergency )
{
    DFBResult ret;
    int       i = 0;

    D_INFO( "MediaTek/System: %s: #%d\n", __FUNCTION__, __LINE__ );

    D_ASSERT( dfb_mt85 != NULL );

    mt85_unmap_debuginfo(dfb_mt85->fd);
#if DFB_SUPPORT_GFX_ADAPTER
    mt85AdapterCmdBufUninit(dfb_mt85->fd);
#endif

    dfb_surface_pool_leave( dfb_mt85->shared->pool );

    if (dfb_mt85->mem)
        munmap( dfb_mt85->mem, dfb_mt85->fix.smem_len );

    for (i = 0; i < mt85_multi_pool.num; i++)
    {
        if (mt85_multi_pool.addr[i])
        {
            munmap(mt85_multi_pool.addr[i], mt85_multi_pool.size[i]);
            mt85_multi_pool.addr[i] = 0;
        }
    }

    if (dfb_config->vt)
    {
        ret = dfb_vt_leave( emergency );
        if (ret)
            return ret;
    }

    close( dfb_mt85->fd );

    D_FREE( dfb_mt85 );
    dfb_mt85 = NULL;

    return DFB_OK;
}

static DFBResult
system_suspend()
{
    return DFB_OK;
}

static DFBResult
system_resume()
{
    return DFB_OK;
}

/******************************************************************************/

static volatile void *
system_map_mmio( unsigned int    offset,
                 int             length )
{
    return NULL;
}

static void
system_unmap_mmio( volatile void  *addr,
                   int             length )
{
}

static int
system_get_accelerator()
{
    return FB_ACCEL_MEDIATEK_85XX;
}

static VideoMode *
system_get_modes()
{
    return NULL;
}

static VideoMode *
system_get_current_mode()
{
    return NULL;
}

static DFBResult
system_thread_init()
{
    return DFB_OK;
}

static bool
system_input_filter( CoreInputDevice *device,
                     DFBInputEvent   *event )
{
    if (dfb_config->vt && dfb_config->vt_switching)
    {
        switch (event->type)
        {
        case DIET_KEYPRESS:
            if (DFB_KEY_TYPE(event->key_symbol) == DIKT_FUNCTION &&
                    event->modifiers == (DIMM_CONTROL | DIMM_ALT))
                return dfb_vt_switch( event->key_symbol - DIKS_F1 + 1 );

            break;

        case DIET_KEYRELEASE:
            if (DFB_KEY_TYPE(event->key_symbol) == DIKT_FUNCTION &&
                    event->modifiers == (DIMM_CONTROL | DIMM_ALT))
                return true;

            break;

        default:
            break;
        }
    }

    return false;
}

static unsigned long
system_video_memory_physical( unsigned int offset )
{
    return dfb_mt85->fix.smem_start + offset;
}

static void *
system_video_memory_virtual( unsigned int offset )
{
    return dfb_mt85->mem + offset;
}

static unsigned int
system_videoram_length()
{
    return dfb_mt85->fix.smem_len;
}

static unsigned long
system_aux_memory_physical( unsigned int offset )
{
    return 0;
}

static void *
system_aux_memory_virtual( unsigned int offset )
{
    return NULL;
}

static unsigned int
system_auxram_length()
{
    return 0;
}

static void
system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
{
}

static void
system_get_deviceid( unsigned int *ret_vendor_id,
                     unsigned int *ret_device_id )
{
}

