/********************************************************************************************
 *     LEGAL DISCLAIMER 
 *
 *     (Header of MediaTek Software/Firmware Release or Documentation)
 *
 *     BY OPENING OR USING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES 
 *     THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE") RECEIVED 
 *     FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON AN "AS-IS" BASIS 
 *     ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES, EXPRESS OR IMPLIED, 
 *     INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
 *     A PARTICULAR PURPOSE OR NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY 
 *     WHATSOEVER WITH RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, 
 *     INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK 
 *     ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
 *     NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S SPECIFICATION 
 *     OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
 *     
 *     BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE LIABILITY WITH 
 *     RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, 
 *     TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE 
 *     FEES OR SERVICE CHARGE PAID BY BUYER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE. 
 *     
 *     THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE WITH THE LAWS 
 *     OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF LAWS PRINCIPLES.  
 ************************************************************************************************/
/*
*
* \par Description
*   The definition of mt85 system
*
*/

#ifndef __MT85__MT85_H__
#define __MT85__MT85_H__

#include <linux/fb.h>

#include <fusion/shm/pool.h>

#include <core/layers.h>
#include <core/screens.h>
#include <core/surface_pool.h>

#include "mt85xx_fb.h"
#include "x_gfx_mt85_comm.h"

//#include "mtk_dfb_cfg.h"

//#include <gfx_cmdque.h>
//#include <gfx_if.h>

#define FB_MEM_SLICE_NUM  15

#include "mt85_surfacemanager.h"
#include "mt85_vt.h"

/// Bitblt option
typedef enum 
{
    E_GFXBLT_TRANSPARENT = 0,
    E_GFXBLT_KEYNOT,
    E_GFXBLT_COLORCHANGE,
    E_GFXBLT_CLIP,
    E_GFXBLT_CFMT_ENA,
    E_GFXBLT_KEYSDSEL
} GFX_BLT_OPT_T;

/// Bitblt option flag : color format translation
#define D_GFXFLAG_CFMT_ENA      (1 << (INT32)E_GFXBLT_CFMT_ENA)

/// Bitblt option flag : transparent
#define D_GFXFLAG_TRANSPARENT   (1 << (INT32)E_GFXBLT_TRANSPARENT)

#ifndef FB_ACCEL_MEDIATEK_85XX
#define FB_ACCEL_MEDIATEK_85XX 8530
#endif

#define MT85_PRIMARY_SUPPORTED_OPTIONS (DLOP_ALPHACHANNEL | DLOP_OPACITY | DLOP_SRC_COLORKEY)

#define MT85_PRIMARY_DEFAULT_WIDTH 1280
#define MT85_PRIMARY_DEFAULT_HEIGHT 720

typedef enum _FB_MEM_TYPE
{
    FBMT_CH1 = 5,
    FBMT_CH2,
    FBMT_LIN,
    FBMT_ONEZONE,
    FBMT_VALLOC   
}FB_MEM_TYPE;

typedef struct _MULTI_POOL
{
    unsigned int    num;
    unsigned int    ref_num;
    bool            iommu_support_flag;
    unsigned char * phys[FB_MEM_SLICE_NUM];
    unsigned char * addr[FB_MEM_SLICE_NUM];    
    unsigned long   size[FB_MEM_SLICE_NUM];
    FB_MEM_TYPE     type[FB_MEM_SLICE_NUM];
    unsigned long   handle[FB_MEM_SLICE_NUM];
}MULTI_POOL;

extern MULTI_POOL mt85_multi_pool;

#define MT85_MAX_OSD_PLANE 3

typedef struct {
     CoreSurfacePool           *pool;

     FusionSHMPoolShared       *shmpool;

     SurfaceManager            *manager;
} MT85Shared;

typedef struct {
     MT85Shared                *shared;

     CoreDFB                   *core;

     int                        fd;
     struct fb_fix_screeninfo   fix;
     void                      *mem;

     struct mt85fb_imagebuffer  imagebuf;
     
     VirtualTerminal           *vt;
} MT85;
typedef enum 
{
    CM_Reserved0 = 0,
    CM_Reserved1 = 1,
    CM_YCbCr_CLUT8 = 2,
    CM_Reserved2 = 3,
    CM_CbYCrY422_DIRECT16 = 4,
    CM_YCbYCr422_DIRECT16 = 5,
    CM_AYCbCr8888_DIRECT32 = 6,
    CM_Reserved3 = 7,
    CM_Reserved4 = 8,
    CM_Reserved5 = 9,
    CM_RGB_CLUT8 = 10,
    CM_RGB565_DIRECT16 = 11,
    CM_ARGB1555_DIRECT16 = 12,
    CM_ARGB4444_DIRECT16 = 13,
    CM_ARGB8888_DIRECT32 = 14,
    CM_Reserved6 = 15
} GFX_COLOR_MODE_T;

typedef struct _GFX_CMDQUE_T
{
    int i4ShortQue;
    int i4QueConfig;
    int i4QueCapacity;
    int i4QueSize;
    int i4PrevIndex;
    int i4ReadIndex;
    int i4WriteIndex;
    int i4QueFlushCount;
    int i4QueIntCount;
    unsigned char *pu1PrevAddr;
    unsigned char *pu1ReadAddr;
    unsigned char *pu1WriteAddr;
    volatile unsigned long long  *pu8QueTop;
    struct _GFX_CMDQUE_T *prNext;
} GFX_CMDQUE_T;

typedef struct {
     /* file descriptor from /dev/fb0 */
     int                    fd;

#if 0 /*yjg*/
     /* mapping of shared command queue control structure and DMA region */
     volatile GFX_CMDQUE_T *shm_cmd_que;
     volatile unsigned long long       *cmd_que_buf;
#endif

     int num_plane;
     
     CoreScreen              *screen;     

     CoreLayer *layer1;
     CoreLayer *layer2;
     CoreLayer *layer3;     
} MT85DriverData;

typedef enum {
     MT85_LAYER_OSD1,
     MT85_LAYER_OSD2,
     MT85_LAYER_OSD3,
     MT85_LAYER_VDP1,
     MT85_LAYER_VDP2
} MT85LayerID;


typedef struct {
     MT85LayerID            layer;
} MT85LayerData;

extern MT85 *dfb_mt85;

extern const SurfacePoolFuncs   mt85SurfacePoolFuncs;
extern /*const*/ DisplayLayerFuncs  mt85PrimaryLayerFuncs;
extern /*const*/ ScreenFuncs        mt85PrimaryScreenFuncs;

#if DFB_SUPPORT_GFX_ADAPTER

// max allowed number of commands in buffer
#if !(GFX_DRV_CMDBUF_DYNAMIC_SIZE)
#define MAX_BUFF_SIZE  256
#endif

#define ASSERT(x)		//((x) ? (void)0 : Assert(#x, __FILE__, (INT32)__LINE__))

typedef struct _MT85_ADAPTER_CMDBUF_T
{
    UINT32                  hGfxCmdBufSema;          // /< GFX Cmd Buf Same
    BOOL                    fg_compress;             //  for WT
    UINT32                  u4Private;              ///< private data, used to verify if cmd buf is valid
    UINT32                  u4GfxPriority;           ///< priority
    GFX_LINUX_CMDBUF_T      *prGfxMwCmdBuf;
} MT85_ADAPTER_CMDBUF_T;
#endif


#endif

