/*
 * jpeghwdef.h
 *
 * Copyright (C) 1991-1997, Thomas G. Lane.
 * Modified 1997-2009 by Guido Vollbeding.
 * This file is part of the Independent JPEG Group's software.
 * For conditions of distribution and use, see the accompanying README file.
 *
 * This file provides common declarations for the various JPEG modules.
 * These declarations are considered internal to the JPEG library; most
 * applications using the library shouldn't need to include this file.
 */
/* Declarations for both compression & decompression */
#include "jpeglib.h"
#ifndef __JPEG_HW____
#define __JPEG_HW____
#if JDEC_LIBJPG_SUPPORT_HWDECODE
/* data passed with IMG_NFY_FILL_BUF */
typedef struct
{
    UINT32          u4DataAdd;
    UINT32          u4DataLen;
    BOOL            fgFileEnd;
} JDEC_FILL_BUF_T;

#if !defined (_TYPEDEF_IMG_DRV_SYMBOL_)
#define _TYPEDEF_IMG_DRV_SYMBOL_
typedef enum
{
    IMG_NFY_FILL_BUF,
    IMG_NFY_FINISHED,
    IMG_NFY_ERROR,
    IMG_NFY_STOP_DONE,
    IMG_NFY_ALREADY_STOPPED
} IMG_NFY_STATE_T;

typedef enum
{
    IMG_JPG_DECODE_NORMAL = 0,
    IMG_JPG_DECODE_SAME_PIC,
} IMG_JPG_DECODE_FLAG_E;

typedef enum
{
    IMG_ROTATE_NONE             = 0,    /* no rotation */
    IMG_ROTATE_CW_90            = 1,    /* clockwise  90 degrees  */
    IMG_ROTATE_CW_180           = 2,    /* clockwise 180 degrees  */
    IMG_ROTATE_CW_270           = 3,    /* clockwise 270 degrees  */
    IMG_ROTATE_NONE_WITH_FLIP   = 4,    /* no rotation, with flip */
    IMG_ROTATE_CW_90_WITH_FLIP  = 5,    /* clockwise  90 degrees, with flip */
    IMG_ROTATE_CW_180_WITH_FLIP = 6,    /* clockwise 180 degrees, with flip */
    IMG_ROTATE_CW_270_WITH_FLIP = 7     /* clockwise 270 degrees, with flip */
} IMG_ROTATE_T;
/* decode quality factor */
typedef enum
{
    IMG_QUALITY_FACTOR_NORMAL = 0,
    IMG_QUALITY_FACTOR_LOW,
    IMG_QUALITY_FACTOR_FAST
} IMG_QUALITY_FACTOR_T;

typedef enum _GFX_COLORMODE_T
{
    GFX_COLORMODE_AYCbCr_CLUT2  = 0,
    GFX_COLORMODE_AYCbCr_CLUT4  = 1,
    GFX_COLORMODE_AYCbCr_CLUT8  = 2,
    GFX_COLORMODE_CbYCrY_16     = 3,
    GFX_COLORMODE_YCbYCr_16     = 4,
    GFX_COLORMODE_AYCbCr_D8888  = 5,
    GFX_COLORMODE_ARGB_CLUT2    = 6,
    GFX_COLORMODE_ARGB_CLUT4    = 7,
    GFX_COLORMODE_ARGB_CLUT8    = 8,
    GFX_COLORMODE_RGB_D565      = 9,
    GFX_COLORMODE_ARGB_D1555    = 10,
    GFX_COLORMODE_ARGB_D4444    = 11,
    GFX_COLORMODE_ARGB_D8888    = 12,
    GFX_COLORMDOE_YUV_420_BLK   = 13,
    GFX_COLORMODE_YUV_420_RS    = 14,
    GFX_COLORMDOE_YUV_422_BLK   = 15,
    GFX_COLORMODE_YUV_422_RS    = 16,
    GFX_COLORMDOE_YUV_444_BLK   = 17,
    GFX_COLORMODE_YUV_444_RS    = 18,
    GFX_COLORMODE_RESERVED = 19

} GFX_COLORMODE_T;

typedef enum
{
   IMG_COMPRESS_NONE=0,
   IMG_COMPRESS_WT=1,
   IMG_COMPRESS_LOSSLESS =2
}IMG_COMPRESS_TYPE_E;
#if UNIFORM_DRV_CALLBACK

typedef struct
{
   UINT32 u4CompressedSize;
   IMG_COMPRESS_TYPE_E     eCompressType;
}IMG_COMPRESS_INF_T;

typedef struct
{
    UINT32 ui4_img_id;
    IMG_NFY_STATE_T e_state;
    union
    {
        void *pv_cache;
        IMG_FILL_BUF_T rFillBuf;
    }rInfo;
    IMG_COMPRESS_INF_T rCompressInfo;
	BOOL b_loss_failed;
}IMG_NFY_PARAM_T;

#else
/* callback function */
typedef VOID (*IMG_NFY_FCT_T)
(
    UINT32          ui4_img_id,         /* the image id which causes this notification */
    VOID            *pv_tag,            /* tag passed to the callback function */
    VOID            *pv_data,           /* data passed with this notification */
    IMG_NFY_STATE_T e_state);           /* notification state */
#endif

typedef INT32 (*VDEC_JPEG_DRV_NOTIFY)(void *pvPrivData);
typedef INT32 (*GPU_DDI_IMG_NOTIFY)(INT32 i4DecResult);

typedef struct
{
  VDEC_JPEG_DRV_NOTIFY pvCallBackFunc;
  void *pvPrivData;
} VDEC_JPEG_DRV_NOTIFY_CB_REG_T;

typedef struct
{
  void *pv_workbuf;
  UINT32 	ui4_buf_size;
}IMG_CMP_WORKBUF_INFO_T;

typedef struct
{
    UINT32          ui4_img_id;         /* the image id decoded */
    VOID            *pv_type_data;      /* frame index */

    UINT32          ui4_src_x;          /* x offset in the source image in pixels */
    UINT32          ui4_src_y;          /* y offset in the source image in pixels */
    UINT32          ui4_src_width;      /* width to be decoded in pixels */
    UINT32          ui4_src_height;     /* height to be decoded in pixels */

    VOID            *pv_dst;            /* destination starting address */
    VOID            *pv_dst2;           /* additional dst starting address */
    UINT32          ui4_dst_x;          /* x offset in the destination in pixels */
    UINT32          ui4_dst_y;          /* y offset in the destination in pixels */
    UINT32          ui4_dst_width;      /* expected output width in pixels */
    UINT32          ui4_dst_height;     /* expected output height in pixels */
    UINT32          ui4_dst_pitch;      /* pitch of the destination image */
    UINT32          ui4_dst2_pitch;     /* pitch of the additional destination image */
    GFX_COLORMODE_T e_dst_cm;           /* destination color mode */

    IMG_ROTATE_T    e_rotate;           /* rotation option */
    IMG_QUALITY_FACTOR_T e_quality;     /* quality factor */ //[20081014] BDP00013841
    BOOL            b_compressed;       /* PNG WT*/
    BOOL            b_directdecodemode; /* for call png decode from user space directly */
#if 1
    IMG_COMPRESS_TYPE_E            e_compressed;       /* PNG WT*/ //b_compressed
    VOID              *pv_losslessinfo;            /* destination starting address */
	UINT32            ui4_losslessinfo_size;
   UINT32          ui4_dst_len; //dst buffer len.
#endif
    IMG_CMP_WORKBUF_INFO_T  t_workbuf;

} IMG_DECODE_T;

/// driver callback function prototypetypedef
typedef VOID (*x_drv_cb_nfy_fct)(VOID* pv_cb_info,    ///< callback information structure, define by each driver module
VOID* pv_nfy_tag     ///< notify tag, this should be the same with pv_nty_tag in DRV_CB_REG_INFO_T
);
#define MNG_FRAME_SUPPORT 1
/* data passed with IMG_SET_TYPE_FRM_START */
typedef struct
{
    UINT32          ui4_img_id;         /* the image id decoded */
    VOID            *pv_img_buf;        /* image data */
    UINT32          ui4_img_size;       /* image size */
    VOID            *pv_aux_cache;      /* auxiliary cache data */
    #if UNIFORM_DRV_CALLBACK
    x_drv_cb_nfy_fct pf_func;
    #else
    IMG_NFY_FCT_T   pf_func;            /* callback function */
    #endif
    VOID            *pv_tag;            /* tag passed to the callback function */
    IMG_JPG_DECODE_FLAG_E          e_jpg_flag;     /* IMG_JPG_DECODE_FLAG_E */
    GPU_DDI_IMG_NOTIFY pvCallBack;
    BOOL            fgGpuOn;
    BOOL            fgVdecPath;
    void *          hJfifInst;
    VDEC_JPEG_DRV_NOTIFY_CB_REG_T* prVdecCallback;
    UINT32         ui4_img_position;
    BOOL             fgKernelImgData;
#if MNG_FRAME_SUPPORT
	BOOL           fgMngFrame;
#endif
} IMG_FRM_START_T;

#include <linux/ioctl.h>
#define JDEC_SET_TYPE_FRM_START _IOWR ('c', 1, IMG_FRM_START_T)
#define JDEC_SET_TYPE_DECODE    _IOWR ('c', 2, IMG_DECODE_T)
#define JDEC_SET_TYPE_FRM_END   _IOWR ('c', 3, UINT32)
#define JDEC_SET_TYPE_SRCRESUME _IOWR ('c', 4, JDEC_FILL_BUF_T)
#define JDEC_SET_TYPE_DEC_INFO  _IOWR ('c', 5, IMG_DECODE_T)
#define JDEC_GET_DEC_PROGRESS   _IOWR ('c', 6, UINT32)
#define JDEC_SET_TYPE_STOP      _IOWR ('c', 7, UINT32)
#endif //_TYPEDEF_IMG_DRV_SYMBOL_
#endif //JDEC_LIBJPG_SUPPORT_HWDECODE

#endif
